from frasco_admin import AdminBlueprint
from frasco import current_app, current_context, abort, request, redirect, url_for
from frasco_models.admin import create_model_admin_blueprint
from frasco_forms.form import wtfields
import inflection


def create_admin_blueprint(users):
    ucol = users.options['username_column']
    ecol = users.options['email_column']

    list_columns = [ucol]
    form_fields = [ucol]
    search_query_default_field = [ucol]
    if ucol != ecol:
        list_columns.append(ecol)
        form_fields.append(ecol)
        search_query_default_field.append(ecol)
    list_columns.extend([
        ('signup_at', 'Signup date'),
        ('last_login_at', 'Last login date')])
    form_fields.append(('password', wtfields.PasswordField()))

    bp = create_model_admin_blueprint("users", __name__, users.model, title="Users", menu="Users",
        icon="fa-users", with_create=False, can_create=True, with_edit=False, can_edit=True,
        search_query_default_field=search_query_default_field, list_columns=list_columns,
        form_fields=form_fields)

    @bp.view("/create", template="admin/models_default/create.html", methods=['GET', 'POST'])
    def create():
        form = bp.get_form_class()()
        current_context['form'] = form
        if form.validate_on_submit():
            user = users.signup(form=form, login_user=False)
            return redirect(url_for('.edit', id=user.id))

    @bp.view("/<id>", template="admin/models_default/edit.html", methods=['GET', 'POST'])
    def edit(id):
        user = users.query.get_or_404(id)
        form = bp.get_form_class()(obj=user)
        current_context['obj'] = user
        current_context['admin_section_title'] = "Edit user #%s" % user.id
        current_context['form'] = form
        current_context['edit_actions'] = [
            ('Reset password', '.reset_password', {}),
            ('Delete user', '.delete', {'style': 'danger'})
        ]
        if form.validate_on_submit():
            users.update_user_from_form(user, form)

    @bp.route("/<id>/reset-password")
    def reset_password(id):
        user = users.query.get_or_404(id)
        users.gen_reset_password_token(user)
        return redirect(url_for('.edit', id=id))

    return bp


from frasco import (Blueprint, with_actions, redirect, request, current_app,\
                    url_for, flash, current_context, pass_feature, session,\
                    populate_obj, ContextExitException)
from frasco.expression import compile_expr


bp = Blueprint("users", __name__, template_folder="templates")


def make_redirect_url(value):
    if value.startswith('http://') or value.startswith('https://'):
        return value
    return url_for(value)


@bp.view('/login', template="users/login.html")
@with_actions([{"form": "LoginForm"}])
@with_actions("form_submitted", ["users.login"])
@pass_feature("users")
def login(users):
    redirect_url = request.args.get("next") or make_redirect_url(users.options["redirect_after_login"])
    if users.logged_in() or request.method == "POST":
        return redirect(redirect_url)


@bp.view('/logout')
@with_actions(["users.logout"])
def logout():
    redirect_to = current_app.features.users.options["redirect_after_logout"]
    if redirect_to:
        return redirect(make_redirect_url(redirect_to))


@bp.view('/signup', template="users/signup.html")
@with_actions([
    {"form": {"name": "SignupForm", "obj": "$session[oauth_user_defaults]", "validate_on_submit": False}}])
@pass_feature("users")
def signup(users):
    if request.method == "GET" and not "oauth" in request.args:
        # signup was accessed directly so we ensure that oauth
        # params stored in session are cleaned. this can happen
        # if a user started to login using an oauth provider but
        # didn't complete the process
        session.pop("oauth_user_defaults", None)
        session.pop("oauth_user_attrs", None)

    current_context["must_provide_password"] = "oauth_user_attrs" not in session \
        or users.options["oauth_must_provide_password"]

    redirect_url = request.args.get("next") or make_redirect_url(users.options["redirect_after_signup"])
    if users.logged_in():
        return redirect(redirect_url)
        
    allow_signup = users.options["allow_signup"]
    if users.options["oauth_signup_only"] and "oauth_user_attrs" not in session:
        allow_signup = False
    if not allow_signup:
        if users.options["signup_disallowed_message"]:
            flash(users.options["signup_disallowed_message"], "error")
        return redirect(url_for(users.options.get("redirect_after_signup_disallowed") or\
            "users.login", next=request.args.get("next")))

    if current_context["form"].is_submitted() and current_context["form"].validate():
        user = users.model()
        if "oauth_user_defaults" in session:
            populate_obj(user, session["oauth_user_defaults"] or {})
        if users.options['require_code_on_signup'] and 'code' in current_context['form'] and\
          not users.check_signup_code(current_context['form'].code.data):
            if users.options['bad_signup_code_message']:
                flash(users.options['bad_signup_code_message'], 'error')
            return redirect(url_for('users.signup', next=request.args.get('next')))
        users.signup(user, form=current_context["form"],
            must_provide_password=current_context["must_provide_password"], **session.get("oauth_user_attrs", {}))
        session.pop("oauth_user_defaults", None)
        session.pop("oauth_user_attrs", None)
        return redirect(redirect_url)


@bp.route('/signup/oauth')
@pass_feature("users", "models")
def oauth_signup(users, models):
    if "oauth_user_attrs" not in session or users.options["oauth_must_signup"]:
        oauth = 1 if users.options["oauth_must_signup"] else 0
        return redirect(url_for(".signup", oauth=oauth, next=request.args.get("next")))

    signup_url = url_for(".signup", oauth=1, next=request.args.get("next"))
    user = users.model()
    populate_obj(user, session["oauth_user_defaults"] or {})
    if not users.validate_signuping_user(user, flash_messages=False, raise_error=False):
        return redirect(signup_url)

    try:
        users.signup(user, must_provide_password=False, **session["oauth_user_attrs"])
    except ContextExitException:
        return redirect(signup_url)
    del session["oauth_user_defaults"]
    del session["oauth_user_attrs"]
    return redirect(request.args.get("next") or make_redirect_url(users.options["redirect_after_login"]))


@bp.view('/login/reset-password', template="users/send_reset_password.html")
@with_actions([{"form": "SendResetPasswordForm"}])
@with_actions("form_submitted", ["users.gen_reset_password_token"])
def send_reset_password():
    msg = current_app.features.users.options["reset_password_token_success_message"]
    redirect_to = current_app.features.users.options["redirect_after_reset_password_token"]
    if request.method == "POST":
        if msg:
            flash(msg, "success")
        if redirect_to:
            return redirect(make_redirect_url(redirect_to))


@bp.view('/login/reset-password/<token>', methods=('GET', 'POST'), template="users/reset_password.html")
@with_actions([{"form": "ResetPasswordForm"}])
@with_actions("form_submitted", ["users.reset_password"])
def reset_password(token):
    msg = current_app.features.users.options["reset_password_success_message"]
    redirect_to = current_app.features.users.options["redirect_after_reset_password"]
    if request.method == "POST":
        if msg:
            flash(msg, "success")
        if redirect_to:
            return redirect(make_redirect_url(redirect_to))



from setuptools import setup, find_packages


setup(
    name='frasco-users',
    version='0.5.1',
    url='http://github.com/frascoweb/frasco-users',
    license='MIT',
    author='Maxime Bouroumeau-Fuseau',
    author_email='maxime.bouroumeau@gmail.com',
    description="Users management for Frasco",
    packages=find_packages(),
    package_data={
        'frasco_users': [
            'templates/users/*.html',
            'emails/users/*.txt',
            'admin/templates/admin/users/*.html']
    },
    zip_safe=False,
    platforms='any',
    install_requires=[
        'frasco',
        'frasco-forms>=0.4',
        'frasco-models>=0.2',
        'Flask-Login>=0.3.0',
        'Flask-Bcrypt>=0.6.0',
        'Flask-OAuthlib>=0.9.0'
    ]
)
